@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -21,15 +21,16 @@
 
 #include "gui/serverdialog.h"
 
+#include "chatlogger.h"
 #include "client.h"
 #include "configuration.h"
 #include "gui.h"
 #include "log.h"
 #include "main.h"
 
+#include "gui/customserverdialog.h"
 #include "gui/okdialog.h"
 #include "gui/sdlinput.h"
-#include "gui/theme.h"
 
 #include "gui/widgets/button.h"
 #include "gui/widgets/dropdown.h"
@@ -41,10 +42,11 @@
 
 #include "net/net.h"
 
+#include "resources/theme.h"
+
 #include "utils/gettext.h"
 #include "utils/stringutils.h"
 #include "utils/xml.h"
-#include "widgets/dropdown.h"
 
 #include <guichan/font.hpp>
 
@@ -114,16 +116,6 @@ void ServersListModel::setVersionString(int index, const std::string &version)
     }
 }
 
-std::string TypeListModel::getElementAt(int elementIndex)
-{
-    if (elementIndex == 0)
-        return "TmwAthena";
-    else if (elementIndex == 1)
-        return "ManaServ";
-    else
-        return "Unknown";
-}
-
 class ServersListBox : public ListBox
 {
 public:
@@ -198,21 +190,10 @@ ServerDialog::ServerDialog(ServerInfo *serverInfo, const std::string &dir):
     mDownloadStatus(DOWNLOADING_PREPARING),
     mDownloadProgress(-1.0f),
     mServers(ServerInfos()),
-#ifndef MANASERV_SUPPORT
-    mManaservServers(ServerInfos()),
-#endif
     mServerInfo(serverInfo)
 {
     setWindowName("ServerDialog");
 
-    Label *serverLabel = new Label(_("Server:"));
-    Label *portLabel = new Label(_("Port:"));
-#ifdef MANASERV_SUPPORT
-    Label *typeLabel = new Label(_("Server type:"));
-#endif
-    mServerNameField = new TextField(mServerInfo->hostname);
-    mPortField = new TextField(toString(mServerInfo->port));
-
     loadCustomServers();
 
     mServersListModel = new ServersListModel(&mServers, this);
@@ -222,48 +203,27 @@ ServerDialog::ServerDialog(ServerInfo *serverInfo, const std::string &dir):
     ScrollArea *usedScroll = new ScrollArea(mServersList);
     usedScroll->setHorizontalScrollPolicy(gcn::ScrollArea::SHOW_NEVER);
 
-    mTypeListModel = new TypeListModel();
-    mTypeField = new DropDown(mTypeListModel);
-    mTypeField->setSelected((serverInfo->type == ServerInfo::MANASERV) ?
-                            1 : 0);
-
     mDescription = new Label(std::string());
+    mDownloadText = new Label(std::string());
 
     mQuitButton = new Button(_("Quit"), "quit", this);
     mConnectButton = new Button(_("Connect"), "connect", this);
-    mManualEntryButton = new Button(_("Custom Server"), "addEntry", this);
+    mManualEntryButton = new Button(_("Add custom Server..."), "addEntry", this);
+    mModifyButton = new Button(_("Modify..."), "modify", this);
     mDeleteButton = new Button(_("Delete"), "remove", this);
 
-    mServerNameField->setActionEventId("connect");
-    mPortField->setActionEventId("connect");
-
-    mServerNameField->addActionListener(this);
-    mPortField->addActionListener(this);
-    mManualEntryButton->addActionListener(this);
+    mServersList->setActionEventId("connect");
     mServersList->addSelectionListener(this);
     usedScroll->setVerticalScrollAmount(0);
 
-    place(0, 0, serverLabel);
-    place(1, 0, mServerNameField, 4).setPadding(3);
-    place(0, 1, portLabel);
-    place(1, 1, mPortField, 4).setPadding(3);
-#ifdef MANASERV_SUPPORT
-    place(0, 2, typeLabel);
-    place(1, 2, mTypeField, 4).setPadding(3);
-    place(0, 3, usedScroll, 5, 5).setPadding(3);
-    place(0, 8, mDescription, 5);
-    place(0, 9, mManualEntryButton);
-    place(1, 9, mDeleteButton);
-    place(3, 9, mQuitButton);
-    place(4, 9, mConnectButton);
-#else
-    place(0, 2, usedScroll, 5, 5).setPadding(3);
-    place(0, 7, mDescription, 5);
-    place(0, 8, mManualEntryButton);
-    place(1, 8, mDeleteButton);
-    place(3, 8, mQuitButton);
-    place(4, 8, mConnectButton);
-#endif
+    place(0, 0, usedScroll, 6, 5).setPadding(3);
+    place(0, 5, mDescription, 5);
+    place(0, 6, mDownloadText, 5);
+    place(0, 7, mManualEntryButton);
+    place(1, 7, mModifyButton);
+    place(2, 7, mDeleteButton);
+    place(4, 7, mQuitButton);
+    place(5, 7, mConnectButton);
 
     // Make sure the list has enough height
     getLayout().setRowHeight(3, 80);
@@ -289,21 +249,11 @@ ServerDialog::ServerDialog(ServerInfo *serverInfo, const std::string &dir):
 
     loadWindowState();
 
-    setFieldsReadOnly(true);
     mServersList->setSelected(0); // Do this after for the Delete button
+
     setVisible(true);
 
-    if (mServerNameField->getText().empty())
-    {
-        mServerNameField->requestFocus();
-    }
-    else
-    {
-        if (mPortField->getText().empty())
-            mPortField->requestFocus();
-        else
-            mConnectButton->requestFocus();
-    }
+    mServersList->requestFocus();
 
     downloadServerList();
 }
@@ -317,7 +267,6 @@ ServerDialog::~ServerDialog()
         mDownload = 0;
     }
     delete mServersListModel;
-    delete mTypeListModel;
 }
 
 void ServerDialog::action(const gcn::ActionEvent &event)
@@ -325,16 +274,21 @@ void ServerDialog::action(const gcn::ActionEvent &event)
     if (event.getId() == "ok")
     {
         // Give focus back to the server dialog.
-        mServerNameField->requestFocus();
+        mServersList->requestFocus();
     }
     else if (event.getId() == "connect")
     {
+        int index = mServersList->getSelected();
+
         // Check login
-        if (mServerNameField->getText().empty()
-            || mPortField->getText().empty())
+        if (index < 0
+#ifndef ENABLE_MANASERV
+            || mServersListModel->getServer(index).type == ServerInfo::MANASERV
+#endif
+        )
         {
             OkDialog *dlg = new OkDialog(_("Error"),
-                _("Please type both the address and the port of a server."));
+                _("Please select a valid server."));
             dlg->addActionListener(this);
         }
         else
@@ -342,26 +296,22 @@ void ServerDialog::action(const gcn::ActionEvent &event)
             mDownload->cancel();
             mQuitButton->setEnabled(false);
             mConnectButton->setEnabled(false);
+            mDeleteButton->setEnabled(false);
+            mManualEntryButton->setEnabled(false);
+            mModifyButton->setEnabled(false);
 
-            mServerInfo->hostname = mServerNameField->getText();
-            mServerInfo->port = (short) atoi(mPortField->getText().c_str());
-            switch (mTypeField->getSelected())
-            {
-                case 0:
-                    mServerInfo->type = ServerInfo::TMWATHENA;
-                    break;
-                case 1:
-                    mServerInfo->type = ServerInfo::MANASERV;
-                    break;
-                default:
-                    mServerInfo->type = ServerInfo::UNKNOWN;
-            }
+            const ServerInfo &serverInfo = mServersListModel->getServer(index);
+            mServerInfo->hostname = serverInfo.hostname;
+            mServerInfo->name = serverInfo.name;
+            mServerInfo->port = serverInfo.port;
+            mServerInfo->type = serverInfo.type;
 
             // Save the selected server
             mServerInfo->save = true;
-
             saveCustomServers(*mServerInfo);
 
+            chatLogger->setServerName(mServerInfo->hostname);
+
             Client::setState(STATE_CONNECT_SERVER);
         }
     }
@@ -372,7 +322,23 @@ void ServerDialog::action(const gcn::ActionEvent &event)
     }
     else if (event.getId() == "addEntry")
     {
-        setFieldsReadOnly(false);
+        // Add a custom server: It will delete itself using guichan logic.
+        new CustomServerDialog(this);
+    }
+    else if (event.getId() == "modify")
+    {
+        int index = mServersList->getSelected();
+        // Check whether a server is selected.
+        if (index < 0)
+        {
+            OkDialog *dlg = new OkDialog(_("Error"),
+                _("Please select a custom server."));
+            dlg->addActionListener(this);
+        }
+        else
+        {
+            new CustomServerDialog(this, index);
+        }
     }
     else if (event.getId() == "remove")
     {
@@ -404,27 +370,26 @@ void ServerDialog::valueChanged(const gcn::SelectionEvent &)
     if (index == -1)
     {
         mDeleteButton->setEnabled(false);
+        mModifyButton->setEnabled(false);
         return;
     }
 
     // Update the server and post fields according to the new selection
     const ServerInfo &myServer = mServersListModel->getServer(index);
     mDescription->setCaption(myServer.description);
-    mServerNameField->setText(myServer.hostname);
-    mPortField->setText(toString(myServer.port));
-    switch (myServer.type)
-    {
-        case ServerInfo::TMWATHENA:
-        case ServerInfo::UNKNOWN:
-            mTypeField->setSelected(0);
-            break;
-        case ServerInfo::MANASERV:
-            mTypeField->setSelected(1);
-            break;
-    }
-    setFieldsReadOnly(true);
 
     mDeleteButton->setEnabled(myServer.save);
+    mModifyButton->setEnabled(myServer.save);
+}
+
+void ServerDialog::mouseClicked(gcn::MouseEvent &mouseEvent)
+{
+    if (mouseEvent.getSource() == mServersList &&
+        isDoubleClick(mServersList->getSelected()))
+    {
+        action(gcn::ActionEvent(mConnectButton,
+                                mConnectButton->getActionEventId()));
+    }
 }
 
 void ServerDialog::logic()
@@ -435,64 +400,39 @@ void ServerDialog::logic()
         {
             mDownloadStatus = DOWNLOADING_OVER;
 
-            mDescription->setCaption(std::string());
+            mDescription->setCaption(mServers[0].description);
+            mDownloadText->setCaption(std::string());
         }
         else if (mDownloadStatus == DOWNLOADING_IN_PROGRESS)
         {
-            mDescription->setCaption(strprintf(_("Downloading server list..."
+            mDownloadText->setCaption(strprintf(_("Downloading server list..."
                                                  "%2.2f%%"),
-                                     mDownloadProgress * 100));
+                                      mDownloadProgress * 100));
         }
         else if (mDownloadStatus == DOWNLOADING_IDLE)
         {
-            mDescription->setCaption(_("Waiting for server..."));
+            mDownloadText->setCaption(_("Waiting for server..."));
         }
         else if (mDownloadStatus == DOWNLOADING_PREPARING)
         {
-            mDescription->setCaption(_("Preparing download"));
+            mDownloadText->setCaption(_("Preparing download"));
         }
         else if (mDownloadStatus == DOWNLOADING_ERROR)
         {
-            mDescription->setCaption(_("Error retreiving server list!"));
+            mDownloadText->setCaption(_("Error retreiving server list!"));
         }
     }
 
     Window::logic();
 }
 
-void ServerDialog::setFieldsReadOnly(bool readOnly)
-{
-    if (!readOnly)
-    {
-        mDescription->setCaption(std::string());
-        mServersList->setSelected(-1);
-
-        mServerNameField->setText(std::string());
-#ifdef MANASERV_SUPPORT
-        mPortField->setText(std::string());
-#else
-        mPortField->setText(std::string("6901"));
-#endif
-
-        mServerNameField->requestFocus();
-    }
-
-    mManualEntryButton->setEnabled(readOnly);
-    mDeleteButton->setEnabled(false);
-    mDescription->setVisible(readOnly);
-
-    mServerNameField->setEnabled(!readOnly);
-    mPortField->setEnabled(!readOnly);
-    mTypeField->setEnabled(!readOnly);
-}
-
 void ServerDialog::downloadServerList()
 {
     // Try to load the configuration value for the onlineServerList
-    std::string listFile = branding.getValue("onlineServerList", std::string());
+    std::string listFile = branding.getStringValue("onlineServerList");
 
     if (listFile.empty())
-        listFile = config.getValue("onlineServerList", std::string());
+        listFile = config.getStringValue("onlineServerList");
 
     // Fall back to manasource.org when neither branding nor config set it
     if (listFile.empty())
@@ -534,7 +474,11 @@ void ServerDialog::loadServers()
         server.type = ServerInfo::parseType(type);
 
         // Ignore unknown server types
-        if (server.type == ServerInfo::UNKNOWN)
+        if (server.type == ServerInfo::UNKNOWN
+#ifndef ENABLE_MANASERV
+            || server.type == ServerInfo::MANASERV
+#endif
+        )
         {
             logger->log("Ignoring server entry with unknown type: %s",
                         type.c_str());
@@ -581,24 +525,23 @@ void ServerDialog::loadServers()
         MutexLocker lock(&mMutex);
         // Add the server to the local list if it's not already present
         bool found = false;
-        for (unsigned int i = 0; i < mServers.size(); i++)
+        int i = 0;
+        for (ServerInfos::iterator it = mServers.begin(); it != mServers.end();
+             ++it)
         {
-            if (mServers[i] == server)
+            if (*it == server)
             {
                 // Use the name listed in the server list
-                mServers[i].name = server.name;
-                mServers[i].version = server.version;
+                (*it).name = server.name;
+                (*it).version = server.version;
                 mServersListModel->setVersionString(i, version);
                 found = true;
                 break;
             }
+            ++i;
         }
 
-#ifdef MANASERV_SUPPORT
         if (!found)
-#else
-        if (!found && server.type != ServerInfo::MANASERV)
-#endif
             mServers.push_back(server);
     }
 }
@@ -608,16 +551,20 @@ void ServerDialog::loadCustomServers()
     for (int i = 0; i < MAX_SERVERLIST; ++i)
     {
         const std::string index = toString(i);
-        const std::string nameKey = "MostUsedServerName" + index;
+        const std::string nameKey = "MostUsedServerDescName" + index;
+        const std::string hostNameKey = "MostUsedServerName" + index;
         const std::string typeKey = "MostUsedServerType" + index;
         const std::string portKey = "MostUsedServerPort" + index;
+        const std::string descriptionKey = "MostUsedServerDescription" + index;
 
         ServerInfo server;
-        server.hostname = config.getValue(nameKey, "");
+        server.name = config.getValue(nameKey, "");
+        server.hostname = config.getValue(hostNameKey, "");
         server.type = ServerInfo::parseType(config.getValue(typeKey, ""));
 
         const int defaultPort = defaultPortForServerType(server.type);
         server.port = (unsigned short) config.getValue(portKey, defaultPort);
+        server.description = config.getValue(descriptionKey, "");
 
         // Stop on the first invalid server
         if (!server.isValid())
@@ -625,80 +572,69 @@ void ServerDialog::loadCustomServers()
 
         server.save = true;
 
-#ifdef MANASERV_SUPPORT
         mServers.push_back(server);
-#else
-        if (server.type == ServerInfo::MANASERV)
-            mManaservServers.push_back(server);
-        else
-            mServers.push_back(server);
-#endif
     }
 }
 
-void ServerDialog::saveCustomServers(const ServerInfo &currentServer)
+void ServerDialog::saveCustomServers(const ServerInfo &currentServer, int index)
 {
+    ServerInfos::iterator it, it_end = mServers.end();
+
     // Make sure the current server is mentioned first
     if (currentServer.isValid())
     {
-        ServerInfos::iterator i, i_end = mServers.end();
-        for (i = mServers.begin(); i != i_end; ++i)
+        if (index > -1)
         {
-            if (*i == currentServer)
+            mServers[index] = currentServer;
+        }
+        else
+        {
+            for (it = mServers.begin(); it != it_end; ++it)
             {
-                mServers.erase(i);
-                break;
+                if (*it == currentServer)
+                {
+                    mServers.erase(it);
+                    break;
+                }
             }
+            mServers.push_front(currentServer);
         }
-        mServers.insert(mServers.begin(), currentServer);
     }
 
     int savedServerCount = 0;
 
-    for (unsigned i = 0;
-         i < mServers.size() && savedServerCount < MAX_SERVERLIST; ++i)
+    for (it = mServers.begin(), it_end = mServers.end();
+         it != it_end && savedServerCount < MAX_SERVERLIST; ++it)
     {
-        const ServerInfo &server = mServers.at(i);
+        const ServerInfo &server = *it;
 
         // Only save servers that were loaded from settings
         if (!(server.save && server.isValid()))
             continue;
 
         const std::string index = toString(savedServerCount);
-        const std::string nameKey = "MostUsedServerName" + index;
+        const std::string nameKey = "MostUsedServerDescName" + index;
+        const std::string hostNameKey = "MostUsedServerName" + index;
         const std::string typeKey = "MostUsedServerType" + index;
         const std::string portKey = "MostUsedServerPort" + index;
+        const std::string descriptionKey = "MostUsedServerDescription" + index;
 
-        config.setValue(nameKey, toString(server.hostname));
+        config.setValue(hostNameKey, toString(server.hostname));
         config.setValue(typeKey, serverTypeToString(server.type));
         config.setValue(portKey, toString(server.port));
+        config.setValue(nameKey, server.name);
+        config.setValue(descriptionKey, server.description);
         ++savedServerCount;
     }
 
-#ifndef MANASERV_SUPPORT
-    for (unsigned i = 0;
-         i < mManaservServers.size() && savedServerCount < MAX_SERVERLIST; ++i)
-    {
-        const ServerInfo &server = mManaservServers.at(i);
-
-        // Only save servers that were loaded from settings
-        if (!(server.save && server.isValid()))
-            continue;
-
-        const std::string index = toString(savedServerCount);
-        const std::string nameKey = "MostUsedServerName" + index;
-        const std::string typeKey = "MostUsedServerType" + index;
-        const std::string portKey = "MostUsedServerPort" + index;
-
-        config.setValue(nameKey, toString(server.hostname));
-        config.setValue(typeKey, serverTypeToString(server.type));
-        config.setValue(portKey, toString(server.port));
-        ++savedServerCount;
-    }
-#endif
     // Insert an invalid entry at the end to make the loading stop there
     if (savedServerCount < MAX_SERVERLIST)
         config.setValue("MostUsedServerName" + toString(savedServerCount), "");
+
+    // Restore the correct description
+    if (index < 0)
+        index = 0;
+    mDescription->setCaption(mServers[index].description);
 }
 
 int ServerDialog::downloadUpdate(void *ptr, DownloadStatus status,
