@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -52,7 +52,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
-#elif defined WIN32
+#elif defined _WIN32
 #include "winver.h"
 #elif defined __APPLE__
 #define PACKAGE_VERSION "0.5.3"
@@ -78,6 +78,8 @@
 #define PACKAGE_OS "Linux"
 #elif defined __GNU__
 #define PACKAGE_OS "GNU Hurd"
+#elif defined __HAIKU__
+#define PACKAGE_OS "Haiku"
 #elif defined WIN32 || defined _WIN32 || defined __WIN32__ || defined __NT__ \
     || defined WIN64 || defined _WIN64 || defined __WIN64__
 #define PACKAGE_OS "Windows"
