@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -20,32 +20,37 @@
  */
 
 #include "net/tmwa/beinghandler.h"
+#include "net/tmwa/playerhandler.h"
 
+#include "actorspritemanager.h"
 #include "being.h"
-#include "beingmanager.h"
 #include "client.h"
 #include "effectmanager.h"
+#include "game.h"
 #include "guild.h"
 #include "localplayer.h"
 #include "log.h"
-#include "npc.h"
 #include "party.h"
 #include "playerrelations.h"
 
 #include "net/tmwa/protocol.h"
 
-#include "resources/colordb.h"
+#include "resources/hairdb.h"
+#include "resources/emotedb.h"
 
 #include <iostream>
+#include <cmath>
 
 namespace TmwAthena {
 
-const int EMOTION_TIME = 150;    /**< Duration of emotion icon */
+// Number of pixels where we decide that the position doesn't need to be reset.
+static const float POS_DEST_DIFF_TOLERANCE = 48.0f;
 
 BeingHandler::BeingHandler(bool enableSync):
    mSync(enableSync)
 {
-    static const Uint16 _messages[] = {
+    static const Uint16 _messages[] =
+    {
         SMSG_BEING_VISIBLE,
         SMSG_BEING_MOVE,
         SMSG_BEING_SPAWN,
@@ -73,21 +78,21 @@ BeingHandler::BeingHandler(bool enableSync):
     handledMessages = _messages;
 }
 
-Being *createBeing(int id, short job)
+static Being *createBeing(int id, short job)
 {
-    Being::Type type = Being::UNKNOWN;
+    ActorSprite::Type type = ActorSprite::UNKNOWN;
     if (job <= 25 || (job >= 4001 && job <= 4049))
-        type = Being::PLAYER;
+        type = ActorSprite::PLAYER;
     else if (job >= 46 && job <= 1000)
-        type = Being::NPC;
+        type = ActorSprite::NPC;
     else if (job > 1000 && job <= 2000)
-        type = Being::MONSTER;
+        type = ActorSprite::MONSTER;
     else if (job == 45)
         return NULL; // Skip portals
 
-    Being *being = beingManager->createBeing(id, type, job);
+    Being *being = actorSpriteManager->createBeing(id, type, job);
 
-    if (type == Being::PLAYER || type == Being::NPC)
+    if (type == ActorSprite::PLAYER || type == ActorSprite::NPC)
     {
         MessageOut outMsg(0x0094);
         outMsg.writeInt32(id);//readLong(2));
@@ -96,13 +101,59 @@ Being *createBeing(int id, short job)
     return being;
 }
 
+static void handleMoveMessage(Map *map, Being *dstBeing,
+                              Uint16 srcX, Uint16 srcY,
+                              Uint16 dstX, Uint16 dstY)
+{
+    // Avoid dealing with flawed destination
+    if (map && dstBeing && srcX && srcY && dstX && dstY)
+    {
+        Vector pos = map->getTileCenter(srcX, srcY);
+        Vector dest = map->getTileCenter(dstX, dstY);
+        Vector beingPos = dstBeing->getPosition();
+
+        // Don't set the position as the movement algorithm
+        // can guess it and it would break the animation played,
+        // when we're close enough.
+        if (std::abs(beingPos.x - pos.x) > POS_DEST_DIFF_TOLERANCE
+            || std::abs(beingPos.y - pos.y) > POS_DEST_DIFF_TOLERANCE)
+            dstBeing->setPosition(pos);
+
+        dstBeing->setDestination(dest.x, dest.y);
+    }
+}
+
+static void handlePosMessage(Map *map, Being *dstBeing, Uint16 x, Uint16 y,
+                             Uint8 dir = 0)
+{
+    // Avoid dealing with flawed destination
+    if (map && dstBeing && x && y)
+    {
+        Vector pos =  map->getTileCenter(x, y);
+        Vector beingPos = dstBeing->getPosition();
+        // Don't set the position as the movement algorithm
+        // can guess it and it would break the animation played,
+        // when we're close enough.
+        if (std::abs(beingPos.x - pos.x) > POS_DEST_DIFF_TOLERANCE
+            || std::abs(beingPos.y - pos.y) > POS_DEST_DIFF_TOLERANCE)
+            dstBeing->setPosition(pos);
+
+        // Set also the destination to the desired position.
+        dstBeing->setDestination(pos.x, pos.y);
+
+        if (dir)
+            dstBeing->setDirection(dir);
+    }
+}
+
 void BeingHandler::handleMessage(Net::MessageIn &msg)
 {
-    if (!beingManager)
+    if (!actorSpriteManager)
         return;
 
     int id;
-    short job, speed, gender;
+    short job, gender;
+    float speed;
     Uint16 headTop, headMid, headBottom;
     Uint16 shoes, gloves;
     Uint16 weapon, shield;
@@ -113,22 +164,24 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
     int type, guild;
     Uint16 status;
     Being *srcBeing, *dstBeing;
-    Player *player = 0;
     int hairStyle, hairColor, flag;
 
+    // Prepare useful translation variables
+    Map *map = Game::instance()->getCurrentMap();
+
     switch (msg.getId())
     {
         case SMSG_BEING_VISIBLE:
         case SMSG_BEING_MOVE:
             // Information about a being in range
             id = msg.readInt32();
-            speed = msg.readInt16();
+            speed = (float)msg.readInt16();
             stunMode = msg.readInt16();  // opt1
             statusEffects = msg.readInt16();  // opt2
             statusEffects |= ((Uint32)msg.readInt16()) << 16;  // option
             job = msg.readInt16();  // class
 
-            dstBeing = beingManager->findBeing(id);
+            dstBeing = actorSpriteManager->findBeing(id);
 
             if (!dstBeing)
             {
@@ -145,22 +198,17 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
                     break;
             }
 
-            if (dstBeing->getType() == Being::PLAYER)
-                player = static_cast<Player*>(dstBeing);
-
-            if (msg.getId() == 0x0078)
+            if (msg.getId() == SMSG_BEING_VISIBLE)
             {
                 dstBeing->clearPath();
-                dstBeing->setFrame(0);
-                dstBeing->setWalkTime(tick_time);
                 dstBeing->setAction(Being::STAND);
             }
 
-
             // Prevent division by 0 when calculating frame
-            if (speed == 0) { speed = 150; }
+            if (speed == 0)
+                speed = 150.0f; // In ticks per tile * 10
 
-            dstBeing->setWalkSpeed(Vector(speed, speed, 0));
+            dstBeing->setMoveSpeed(Vector(speed / 10, speed / 10));
             dstBeing->setSubtype(job);
             hairStyle = msg.readInt16();
             weapon = msg.readInt16();
@@ -178,16 +226,13 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
             shoes = msg.readInt16();  // clothes color - "abused" as shoes
             gloves = msg.readInt16();  // head dir - "abused" as gloves
             guild = msg.readInt32();  // guild
-            if (player)
+            if (guild == 0)
             {
-                if (guild == 0)
-                {
-                    player->clearGuilds();
-                }
-                else
-                {
-                    player->addGuild(Guild::getGuild(guild));
-                }
+                dstBeing->clearGuilds();
+            }
+            else
+            {
+                dstBeing->addGuild(Guild::getGuild(guild));
             }
             msg.readInt16();  // guild emblem
             msg.readInt16();  // manner
@@ -195,36 +240,34 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
             msg.readInt8();   // karma
             gender = msg.readInt8();
 
-            if (player)
+            if (dstBeing->getType() == ActorSprite::PLAYER)
             {
-                player->setGender((gender == 0)
-                                  ? GENDER_FEMALE : GENDER_MALE);
+                dstBeing->setGender((gender == 0)
+                                    ? GENDER_FEMALE : GENDER_MALE);
                 // Set these after the gender, as the sprites may be gender-specific
-                player->setSprite(SPRITE_HAIR, hairStyle * -1, ColorDB::get(hairColor));
-                player->setSprite(SPRITE_BOTTOMCLOTHES, headBottom);
-                player->setSprite(SPRITE_TOPCLOTHES, headMid);
-                player->setSprite(SPRITE_HAT, headTop);
-                player->setSprite(SPRITE_SHOE, shoes);
-                player->setSprite(SPRITE_GLOVES, gloves);
-                player->setSprite(SPRITE_WEAPON, weapon, "", true);
-                player->setSprite(SPRITE_SHIELD, shield);
+                dstBeing->setSprite(SPRITE_HAIR, hairStyle * -1,
+                                    hairDB.getHairColor(hairColor));
+                dstBeing->setSprite(SPRITE_BOTTOMCLOTHES, headBottom);
+                dstBeing->setSprite(SPRITE_TOPCLOTHES, headMid);
+                dstBeing->setSprite(SPRITE_HAT, headTop);
+                dstBeing->setSprite(SPRITE_SHOE, shoes);
+                dstBeing->setSprite(SPRITE_GLOVES, gloves);
+                dstBeing->setSprite(SPRITE_WEAPON, weapon, "", true);
+                dstBeing->setSprite(SPRITE_SHIELD, shield);
             }
 
             if (msg.getId() == SMSG_BEING_MOVE)
             {
                 Uint16 srcX, srcY, dstX, dstY;
                 msg.readCoordinatePair(srcX, srcY, dstX, dstY);
-                dstBeing->setAction(Being::STAND);
-                dstBeing->setTileCoords(srcX, srcY);
-                dstBeing->setDestination(dstX, dstY);
+                handleMoveMessage(map, dstBeing, srcX, srcY, dstX, dstY);
             }
             else
             {
                 Uint8 dir;
                 Uint16 x, y;
                 msg.readCoordinates(x, y, dir);
-                dstBeing->setTileCoords(x, y);
-                dstBeing->setDirection(dir);
+                handlePosMessage(map, dstBeing, x, y, dir);
             }
 
             msg.readInt8();   // unknown
@@ -244,16 +287,13 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
              break;
 
         case SMSG_BEING_MOVE2:
+        {
             /*
              * A simplified movement packet, used by the
              * later versions of eAthena for both mobs and
              * players
              */
-            dstBeing = beingManager->findBeing(msg.readInt32());
-
-            Uint16 srcX, srcY, dstX, dstY;
-            msg.readCoordinatePair(srcX, srcY, dstX, dstY);
-            msg.readInt32();  // Server tick
+            dstBeing = actorSpriteManager->findBeing(msg.readInt32());
 
             /*
              * This packet doesn't have enough info to actually
@@ -261,47 +301,44 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
              * we'll just pretend the packet didn't happen
              */
 
-            if (dstBeing)
-            {
-                dstBeing->setAction(Being::STAND);
-                dstBeing->setTileCoords(srcX, srcY);
-                dstBeing->setDestination(dstX, dstY);
-            }
+            if (!dstBeing)
+                break;
 
+            Uint16 srcX, srcY, dstX, dstY;
+            msg.readCoordinatePair(srcX, srcY, dstX, dstY);
+            msg.readInt32();  // Server tick
+            handleMoveMessage(map, dstBeing, srcX, srcY, dstX, dstY);
+        }
             break;
 
         case SMSG_BEING_REMOVE:
             // A being should be removed or has died
             id = msg.readInt32();
-
-            dstBeing = beingManager->findBeing(id);
-
+            dstBeing = actorSpriteManager->findBeing(id);
             if (!dstBeing)
                 break;
 
             // If this is player's current target, clear it.
-            if (dstBeing == player_node->getTarget())
-                player_node->stopAttack();
+            if (dstBeing == local_player->getTarget())
+                local_player->stopAttack();
 
             if (msg.readInt8() == 1)
                 dstBeing->setAction(Being::DEAD);
             else
-                beingManager->destroyBeing(dstBeing);
+                actorSpriteManager->destroy(dstBeing);
 
             break;
 
         case SMSG_BEING_RESURRECT:
             // A being changed mortality status
             id = msg.readInt32();
-
-            dstBeing = beingManager->findBeing(id);
-
+            dstBeing = actorSpriteManager->findBeing(id);
             if (!dstBeing)
                 break;
 
             // If this is player's current target, clear it.
-            if (dstBeing == player_node->getTarget())
-                player_node->stopAttack();
+            if (dstBeing == local_player->getTarget())
+                local_player->stopAttack();
 
             if (msg.readInt8() == 1)
                 dstBeing->setAction(Being::STAND);
@@ -309,25 +346,28 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
             break;
 
         case SMSG_SKILL_DAMAGE:
+        {
             msg.readInt16(); // Skill Id
-            srcBeing = beingManager->findBeing(msg.readInt32());
-            dstBeing = beingManager->findBeing(msg.readInt32());
+            srcBeing = actorSpriteManager->findBeing(msg.readInt32());
+            dstBeing = actorSpriteManager->findBeing(msg.readInt32());
             msg.readInt32(); // Server tick
-            msg.readInt32(); // src speed
+            int attackSpeed = msg.readInt32(); // src speed
             msg.readInt32(); // dst speed
             param1 = msg.readInt32(); // Damage
             msg.readInt16(); // Skill level
             msg.readInt16(); // Div
             msg.readInt8(); // Skill hit/type (?)
+            if (attackSpeed && srcBeing && srcBeing != local_player)
+                srcBeing->setAttackSpeed(attackSpeed);
             if (dstBeing)
                 dstBeing->takeDamage(srcBeing, param1, Being::HIT); // Perhaps a new skill attack type should be created and used?
             if (srcBeing)
-                srcBeing->handleAttack(dstBeing, param1, Being::HIT);
+                srcBeing->handleAttack(dstBeing, param1);
             break;
-
+        }
         case SMSG_BEING_ACTION:
-            srcBeing = beingManager->findBeing(msg.readInt32());
-            dstBeing = beingManager->findBeing(msg.readInt32());
+            srcBeing = actorSpriteManager->findBeing(msg.readInt32());
+            dstBeing = actorSpriteManager->findBeing(msg.readInt32());
             msg.readInt32();   // server tick
             msg.readInt32();   // src speed
             msg.readInt32();   // dst speed
@@ -347,14 +387,12 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
                         dstBeing->takeDamage(srcBeing, param1,
                                 (Being::AttackType)type);
                     if (srcBeing)
-                        srcBeing->handleAttack(dstBeing, param1,
-                                (Being::AttackType)type);
+                        srcBeing->handleAttack(dstBeing, param1);
                     break;
 
                 case 0x02: // Sit
                     if (srcBeing)
                     {
-                        srcBeing->setFrame(0);
                         srcBeing->setAction(Being::SIT);
                     }
                     break;
@@ -362,37 +400,36 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
                 case 0x03: // Stand up
                     if (srcBeing)
                     {
-                        srcBeing->setFrame(0);
                         srcBeing->setAction(Being::STAND);
                     }
                     break;
             }
             break;
 
-        case SMSG_BEING_SELFEFFECT: {
+        case SMSG_BEING_SELFEFFECT:
+        {
             id = (Uint32)msg.readInt32();
-            if (!beingManager->findBeing(id))
+            Being* being = actorSpriteManager->findBeing(id);
+            if (!being)
                 break;
 
             int effectType = msg.readInt32();
-            Being* being = beingManager->findBeing(id);
 
             effectManager->trigger(effectType, being);
-
             break;
         }
 
         case SMSG_BEING_EMOTION:
-            if (!(dstBeing = beingManager->findBeing(msg.readInt32())))
+            if (!(dstBeing = actorSpriteManager->findBeing(msg.readInt32())))
             {
                 break;
             }
 
             if (player_relations.hasPermission(dstBeing, PlayerRelation::EMOTE))
             {
-                // only set emote if one doesnt already exist
-                if (!dstBeing->getEmotion())
-                    dstBeing->setEmote(msg.readInt8(), EMOTION_TIME);
+                const int fx = EmoteDB::get(msg.readInt8())->effect;
+                //TODO: figure out why the -1 is needed
+                effectManager->trigger(fx - 1, dstBeing);
             }
 
             break;
@@ -412,14 +449,11 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
              * 16 bit value will be 0.
              */
 
-            if (!(dstBeing = beingManager->findBeing(msg.readInt32())))
+            if (!(dstBeing = actorSpriteManager->findBeing(msg.readInt32())))
             {
                 break;
             }
 
-            if (dstBeing->getType() == Being::PLAYER)
-                player = static_cast<Player*>(dstBeing);
-
             int type = msg.readInt8();
             int id = 0;
             int id2 = 0;
@@ -437,41 +471,42 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
             switch (type)
             {
                 case 1:     // eAthena LOOK_HAIR
-                    player->setSpriteID(SPRITE_HAIR, id *-1);
+                    dstBeing->setSpriteID(SPRITE_HAIR, id *-1);
                     break;
                 case 2:     // Weapon ID in id, Shield ID in id2
-                    player->setSprite(SPRITE_WEAPON, id, "", true);
-                    player->setSprite(SPRITE_SHIELD, id2);
+                    dstBeing->setSprite(SPRITE_WEAPON, id, "", true);
+                    dstBeing->setSprite(SPRITE_SHIELD, id2);
                     break;
                 case 3:     // Change lower headgear for eAthena, pants for us
-                    player->setSprite(SPRITE_BOTTOMCLOTHES, id);
+                    dstBeing->setSprite(SPRITE_BOTTOMCLOTHES, id);
                     break;
                 case 4:     // Change upper headgear for eAthena, hat for us
-                    player->setSprite(SPRITE_HAT, id);
+                    dstBeing->setSprite(SPRITE_HAT, id);
                     break;
                 case 5:     // Change middle headgear for eathena, armor for us
-                    player->setSprite(SPRITE_TOPCLOTHES, id);
+                    dstBeing->setSprite(SPRITE_TOPCLOTHES, id);
                     break;
                 case 6:     // eAthena LOOK_HAIR_COLOR
-                    player->setSpriteColor(SPRITE_HAIR, ColorDB::get(id));
+                    dstBeing->setSpriteColor(SPRITE_HAIR,
+                                             hairDB.getHairColor(id));
                     break;
                 case 8:     // eAthena LOOK_SHIELD
-                    player->setSprite(SPRITE_SHIELD, id);
+                    dstBeing->setSprite(SPRITE_SHIELD, id);
                     break;
                 case 9:     // eAthena LOOK_SHOES
-                    player->setSprite(SPRITE_SHOE, id);
+                    dstBeing->setSprite(SPRITE_SHOE, id);
                     break;
                 case 10:   // LOOK_GLOVES
-                    player->setSprite(SPRITE_GLOVES, id);
+                    dstBeing->setSprite(SPRITE_GLOVES, id);
                     break;
                 case 11:  // LOOK_CAPE
-                    player->setSprite(SPRITE_CAPE, id);
+                    dstBeing->setSprite(SPRITE_CAPE, id);
                     break;
                 case 12:
-                    player->setSprite(SPRITE_MISC1, id);
+                    dstBeing->setSprite(SPRITE_MISC1, id);
                     break;
                 case 13:
-                    player->setSprite(SPRITE_MISC2, id);
+                    dstBeing->setSprite(SPRITE_MISC2, id);
                     break;
                 default:
                     logger->log("SMSG_BEING_CHANGE_LOOKS: unsupported type: "
@@ -482,13 +517,13 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
             break;
 
         case SMSG_BEING_NAME_RESPONSE:
-            if ((dstBeing = beingManager->findBeing(msg.readInt32())))
+            if ((dstBeing = actorSpriteManager->findBeing(msg.readInt32())))
             {
                 dstBeing->setName(msg.readString(24));
             }
             break;
         case SMSG_PLAYER_GUILD_PARTY_INFO:
-            if ((dstBeing = beingManager->findBeing(msg.readInt32())))
+            if ((dstBeing = actorSpriteManager->findBeing(msg.readInt32())))
             {
                 dstBeing->setPartyName(msg.readString(24));
                 dstBeing->setGuildName(msg.readString(24));
@@ -497,15 +532,17 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
             }
             break;
         case SMSG_BEING_CHANGE_DIRECTION:
-            if (!(dstBeing = beingManager->findBeing(msg.readInt32())))
+        {
+            if (!(dstBeing = actorSpriteManager->findBeing(msg.readInt32())))
             {
                 break;
             }
 
             msg.readInt16(); // unused
-
-            dstBeing->setDirection(msg.readInt8());
-
+            Uint8 dir = msg.readInt8();
+            if (dir)
+                dstBeing->setDirection(dir);
+        }
             break;
 
         case SMSG_PLAYER_UPDATE_1:
@@ -520,7 +557,7 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
                 << 16; // status.options; Aethyra uses this as misc2
             job = msg.readInt16();
 
-            dstBeing = beingManager->findBeing(id);
+            dstBeing = actorSpriteManager->findBeing(id);
 
             if (!dstBeing)
             {
@@ -530,17 +567,19 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
                     break;
             }
 
-            if (dstBeing->getType() == Being::PLAYER)
-                player = static_cast<Player*>(dstBeing);
-
-            if (Party *party = player_node->getParty()){
+            if (Party *party = local_player->getParty()){
                 if (party->isMember(id))
                 {
-                    player->setParty(party);
+                    dstBeing->setParty(party);
                 }
             }
 
-            dstBeing->setWalkSpeed(Vector(speed, speed, 0));
+            // The original speed is ticks per tile * 10
+            if (speed)
+                dstBeing->setMoveSpeed(Vector(speed / 10, speed / 10));
+            else
+                dstBeing->setMoveSpeed(Net::getPlayerHandler()->getDefaultMoveSpeed());
+
             dstBeing->setSubtype(job);
             hairStyle = msg.readInt16();
             weapon = msg.readInt16();
@@ -562,41 +601,40 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
             msg.readInt16();  // manner
             dstBeing->setStatusEffectBlock(32, msg.readInt16());  // opt3
             msg.readInt8();   // karma
-            player->setGender((msg.readInt8() == 0)
+            dstBeing->setGender((msg.readInt8() == 0)
                               ? GENDER_FEMALE : GENDER_MALE);
 
             // Set these after the gender, as the sprites may be gender-specific
-            player->setSprite(SPRITE_WEAPON, weapon, "", true);
-            player->setSprite(SPRITE_SHIELD, shield);
-            //player->setSprite(SPRITE_SHOE, shoes);
-            player->setSprite(SPRITE_BOTTOMCLOTHES, headBottom);
-            player->setSprite(SPRITE_TOPCLOTHES, headMid);
-            player->setSprite(SPRITE_HAT, headTop);
-            //player->setSprite(SPRITE_GLOVES, gloves);
-            //player->setSprite(SPRITE_CAPE, cape);
-            //player->setSprite(SPRITE_MISC1, misc1);
-            //player->setSprite(SPRITE_MISC2, misc2);
-            player->setSprite(SPRITE_HAIR, hairStyle * -1, ColorDB::get(hairColor));
+            dstBeing->setSprite(SPRITE_WEAPON, weapon, "", true);
+            dstBeing->setSprite(SPRITE_SHIELD, shield);
+            //dstBeing->setSprite(SPRITE_SHOE, shoes);
+            dstBeing->setSprite(SPRITE_BOTTOMCLOTHES, headBottom);
+            dstBeing->setSprite(SPRITE_TOPCLOTHES, headMid);
+            dstBeing->setSprite(SPRITE_HAT, headTop);
+            //dstBeing->setSprite(SPRITE_GLOVES, gloves);
+            //dstBeing->setSprite(SPRITE_CAPE, cape);
+            //dstBeing->setSprite(SPRITE_MISC1, misc1);
+            //dstBeing->setSprite(SPRITE_MISC2, misc2);
+            dstBeing->setSprite(SPRITE_HAIR, hairStyle * -1,
+                                hairDB.getHairColor(hairColor));
 
             if (msg.getId() == SMSG_PLAYER_MOVE)
             {
                 Uint16 srcX, srcY, dstX, dstY;
                 msg.readCoordinatePair(srcX, srcY, dstX, dstY);
-                dstBeing->setTileCoords(srcX, srcY);
-                dstBeing->setDestination(dstX, dstY);
+                handleMoveMessage(map, dstBeing, srcX, srcY, dstX, dstY);
             }
             else
             {
                 Uint8 dir;
                 Uint16 x, y;
                 msg.readCoordinates(x, y, dir);
-                dstBeing->setTileCoords(x, y);
-                dstBeing->setDirection(dir);
+                handlePosMessage(map, dstBeing, x, y, dir);
             }
 
             gmstatus = msg.readInt16();
             if (gmstatus & 0x80)
-                player->setGM(true);
+                dstBeing->setGM(true);
 
             if (msg.getId() == SMSG_PLAYER_UPDATE_1)
             {
@@ -619,9 +657,6 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
             msg.readInt8();   // Lv
             msg.readInt8();   // unknown
 
-            dstBeing->setWalkTime(tick_time);
-            dstBeing->setFrame(0);
-
             dstBeing->setStunMode(stunMode);
             dstBeing->setStatusEffectBlock(0, (statusEffects >> 16) & 0xffff);
             dstBeing->setStatusEffectBlock(16, statusEffects & 0xffff);
@@ -641,20 +676,15 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
              */
 
             id = msg.readInt32();
-            if (mSync || id != player_node->getId())
+            if (mSync || id != local_player->getId())
             {
-                dstBeing = beingManager->findBeing(id);
+                dstBeing = actorSpriteManager->findBeing(id);
                 if (dstBeing)
                 {
                     Uint16 x, y;
                     x = msg.readInt16();
                     y = msg.readInt16();
-                    dstBeing->setTileCoords(x, y);
-                    if (dstBeing->getCurrentAction() == Being::WALK)
-                    {
-                        dstBeing->setFrame(0);
-                        dstBeing->setAction(Being::STAND);
-                    }
+                    handlePosMessage(map, dstBeing, x, y);
                 }
             }
             break;
@@ -671,18 +701,18 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
         case SMSG_PLAYER_STATUS_CHANGE:
             // Change in players' flags
             id = msg.readInt32();
-            dstBeing = beingManager->findBeing(id);
+            dstBeing = actorSpriteManager->findBeing(id);
+            if (!dstBeing)
+                break;
+
             stunMode = msg.readInt16();
             statusEffects = msg.readInt16();
             statusEffects |= ((Uint32) msg.readInt16()) << 16;
-            msg.readInt8();
+            msg.readInt8(); // Unused?
 
-            if (dstBeing)
-            {
-                dstBeing->setStunMode(stunMode);
-                dstBeing->setStatusEffectBlock(0, (statusEffects >> 16) & 0xffff);
-                dstBeing->setStatusEffectBlock(16, statusEffects & 0xffff);
-            }
+            dstBeing->setStunMode(stunMode);
+            dstBeing->setStatusEffectBlock(0, (statusEffects >> 16) & 0xffff);
+            dstBeing->setStatusEffectBlock(16, statusEffects & 0xffff);
             break;
 
         case SMSG_BEING_STATUS_CHANGE:
@@ -691,7 +721,7 @@ void BeingHandler::handleMessage(Net::MessageIn &msg)
             id = msg.readInt32();
             flag = msg.readInt8(); // 0: stop, 1: start
 
-            dstBeing = beingManager->findBeing(id);
+            dstBeing = actorSpriteManager->findBeing(id);
             if (dstBeing)
                 dstBeing->setStatusEffect(status, flag);
             break;
